import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart';
import 'package:analyzer_plugin/protocol/protocol_generated.dart';
import 'package:custom_code_analysis_plus/src/logger/log.dart';
import 'package:custom_code_analysis_plus/src/model/error_issue.dart';

AnalysisError codeIssueToAnalysisError(Issue issue, ResolvedUnitResult analysisResult) {
  logUtil.info('parse issue  = $issue');
  return AnalysisError(
    issue.errorSeverity!,
    issue.errorType!,
    Location(
      analysisResult.unit.declaredElement?.source.fullName ?? 'unknown',
      issue.offset!,
      issue.length!,
      issue.line!,
      issue.column!,
      endLine: issue.endLine!,
      endColumn: issue.endColumn!,
    ),
    issue.message ?? '',
    issue.code ?? '',
    correction: issue.correction,
    hasFix: issue.hasFix,
  );
}

List<AnalysisErrorFixes> codeIssueToAnalysisErrorFixes(List<Issue> issueList, ResolvedUnitResult analysisResult) {
  return issueList
      .map(
        (issue) => AnalysisErrorFixes(
          codeIssueToAnalysisError(issue, analysisResult),
          fixes: [
            if (issue.correction != null)
              PrioritizedSourceChange(
                99,
                SourceChange(
                  issue.comment!,
                  edits: [
                    SourceFileEdit(
                      analysisResult.libraryElement.source.fullName,
                      analysisResult.libraryElement.source.contents.modificationTime,
                      edits: [
                        SourceEdit(
                          issue.offset!,
                          issue.length!,
                          issue.replacement ?? '',
                        ),
                      ],
                    ),
                  ],
                  // selection: Position(unitResult.libraryElement.source.fullName, issue.offset),
                ),
              ),
          ],
        ),
      )
      .toList();
}
